# coding: utf-8

import os
import sys
from workspacemanager.utils import *
from workspacemanager.test.utils import fileToStr
import sh
from shutil import copyfile
import re
import json
import getpass
from .setup import getConf

def purge(dir, pattern):
    for f in os.listdir(dir):
        if re.search(pattern, f):
            os.remove(os.path.join(dir, f))

def getDependencies(theProjectDirectory=None, alreadyLocalInstalled=None):
    """
        This funtion return all absolutes paths of all dependencies of the current project    
    """
    # Get all dirs:
    (thisLibPackageDirectory,
    theProjectDirectory,
    theProjectPackageDirectory,
    thisLibName,
    workspacePath,
    theProjectName,
    thePackageName) = getDirs2(theProjectDirectory=theProjectDirectory)

    # filePath:
    filePath = theProjectDirectory + "/local-dependencies.txt"
    
    # Generate dist of all dependencies:
    dependenciesPaths = []
    if alreadyLocalInstalled is None:
        alreadyLocalInstalled = [theProjectName]
    if os.path.isfile(filePath):
        with open(filePath, 'r') as f:
            for line in f:
                line = line.strip()
                # Remove the replacement project:
                if "/" in line:
                    line = line.split("/")[0]
                if len(line) > 0 and line not in alreadyLocalInstalled:
                    alreadyLocalInstalled.append(line)
                    currentDepPath = findProject(workspacePath, line)
                    if os.path.isdir(currentDepPath):
                        dependenciesPaths.append(currentDepPath)
                        dependenciesPaths += getDependencies(theProjectDirectory=currentDepPath, alreadyLocalInstalled=alreadyLocalInstalled)
                    else:
                        print(line + " doesn't exist.")
    return dependenciesPaths
    
def generateDist(theProjectDirectory):
    """
        This function generate the dist of the given project and return the absolute path of the tar.gz
    """
    # We generate the dist of the current project:
    sh.cd(theProjectDirectory)
    sh.python("setup.py", "clean", "--all")
    sh.python("setup.py", "sdist")
    print("Generating " + theProjectDirectory + " dist...")
    # We search the last tar.gz:
    theTarGz = sortedGlob(theProjectDirectory + "/dist/*.gz", sortBy=GlobSortEnum.MTIME, reverse=False)[-1]
    return theTarGz

def generateDists(theProjectDirectory=None, theProjectVenvName=None, alreadyLocalInstalled=None, indent=0):
    """
        This function generate all dist of all dependencies (recursively) and the dist of the current project.
        Return the list of all tar.gz (absolute paths)
    """
    # Get all dirs:
    (thisLibPackageDirectory,
    theProjectDirectory,
    theProjectPackageDirectory,
    thisLibName,
    workspacePath,
    theProjectName,
    thePackageName) = getDirs2(theProjectDirectory=theProjectDirectory)
    # Delete all previous generated tar.gz:
    gatherFolder = theProjectDirectory + "/wm-dist/"
    if not os.path.exists(gatherFolder):
        os.mkdir(gatherFolder)
    purge(gatherFolder, ".*\.tar\.gz")
    # We generate the dist of the current project:
    allTarGz = []
    allTarGz.append(generateDist(theProjectDirectory))
    # We generate tar.gz of all dependencies (recursively):
    allDeps = getDependencies(theProjectDirectory)
    for currentDep in allDeps:
        allTarGz.append(generateDist(currentDep))
    # We gather all tar.gz:
    for currentTarGz in allTarGz:
        copyfile(currentTarGz, gatherFolder + currentTarGz.split("/")[-1])
    # Copy the dist-templates folder:
    conf = getConf(workspacePath)
    if "erase_wm-dist_templates" not in conf:
        conf["erase_wm-dist_templates"] = False
    templatePath = thisLibPackageDirectory + "/dist-templates"
    allTemplateFiles = [f for f in os.listdir(templatePath) if os.path.isfile(os.path.join(templatePath, f))]
    for fileName in allTemplateFiles:
        filePath = templatePath + "/" + fileName
        filePathPaste = gatherFolder + "/" + fileName
        if ".pyc" not in filePathPaste:
            if conf["erase_wm-dist_templates"]:
                copyfile(filePath, filePathPaste)
                print(fileName + " created.")
            elif not os.path.isfile(filePathPaste):
                copyfile(filePath, filePathPaste)
                print(fileName + " created.")
    # Creating the conf file:
    confPath = gatherFolder + '/conf.json'
    if not os.path.isfile(confPath):
        distConf = {"path": "~", "port": "22", "project": theProjectName, "address": "localhost", "venv": getVenvName(theProjectName), "user": getpass.getuser()}
        with open(confPath, 'w') as fp:
            json.dump(distConf, fp)
    # Making the script executable:
    for current in sortedGlob(gatherFolder + "/*.sh"):
        sh.chmod("+x", current)

if __name__ == '__main__':
    generateDists()




# coding: utf-8

import os
import sys
from workspacemanager.utils import *
from workspacemanager.test.utils import fileToStr
import sh



def installDeps(theProjectDirectory=None, theProjectVenvName=None, alreadyLocalInstalled=None, indent=0):
    # Get args:
    argv = argvOptionsToDict()
    if argv is None:
        print("Please check the readme for the command usage.")
        exit()
    
    # Create indentText:
    indentText = "\t" * indent
    
    
    # Get all dirs:
    (thisLibPackageDirectory,
    theProjectDirectory,
    theProjectPackageDirectory,
    thisLibName,
    workspacePath,
    theProjectName,
    thePackageName) = getDirs2(theProjectDirectory=theProjectDirectory)
    
    if theProjectVenvName is None:
        theProjectVenvName = thePackageName + "-venv"

    # filePath:
    filePath = theProjectDirectory + "/local-dependencies.txt"
    if "r" in argv:
        filePath = theProjectDirectory + "/" + argv["r"]
    
    # Install all dependencies:
    if alreadyLocalInstalled is None:
        alreadyLocalInstalled = [theProjectName]
    if os.path.isfile(filePath):
        with open(filePath, 'r') as f:
            for line in f:
                line = line.strip()
                # Remove the replacement project:
                if "/" in line:
                    line = line.split("/")[0]
                if len(line) > 0 and line not in alreadyLocalInstalled:
                    alreadyLocalInstalled.append(line)
                    currentDepPath = findProject(workspacePath, line)
                    if os.path.isdir(currentDepPath):
                        print(indentText + "Installing " + line + "...")
                        sh.cd(currentDepPath)
                        # "pew in " + theProjectVenvName + " python setup.py install"
#                         sh.yes(sh.pew("in", theProjectVenvName, "pip", "uninstall", line.lower())) # pip uninstall workspacemanager
#                         sh.pew("in", theProjectVenvName, "pip", "uninstall", line.lower(), _in="y") # pip uninstall workspacemanager
#                         with sh.contrib.sudo("-H"):
#                             sh.pew("in", theProjectVenvName, "pip", "uninstall", line.lower())
                        sh.pew("in", theProjectVenvName, "python", "setup.py", "install")
                        sh.pew("in", theProjectVenvName, 'pip', 'install', '-r', 'requirements.txt')
                        # Install all dependencies of the current dependency:
                        installDeps(theProjectDirectory=currentDepPath, theProjectVenvName=theProjectVenvName, alreadyLocalInstalled=alreadyLocalInstalled, indent=indent+1)
                    else:
                        print(line + " doesn't exist.")

if __name__ == '__main__':
    installDeps()
















# coding: utf-8
# to execute using python command line and not eclipse :
# cd /home/hayj/Workspace/Python/Organization/WorkspaceManager && sudo python ./workspacemanager/test/venvtest.py

import unittest
import doctest
from workspacemanager import venv
from workspacemanager import generateVenv
from workspacemanager.utils import *
from workspacemanager.test.utils import *
import sh

# The level allow the unit test execution to choose only the top level test 
min = 0
max = 1
assert min <= max


if min <= 0 <= max:
    class DocTest(unittest.TestCase):
        def testDoctests(self):
            """Run doctests"""
            doctest.testmod(venv)

if min <= 1 <= max:
    class Test1(unittest.TestCase):
        def test1(self):
            # Create a fake project:
            theProjectDirectory = createFakeDir()
            
            # Check the fake project:
            assert os.path.isdir(theProjectDirectory) is True
            
            # Get names:
            thePackageName = theProjectDirectory.split('/')[-1].lower()
            venvName = thePackageName + "-venv"
            
            # We check that the venv doesn't exists:
            venvsList = sh.pew("ls").split()
            print("Available venvs:")
            print(venvsList)
            self.assertFalse(venvName in venvsList)
            
            # Generate the venv:
            generateVenv(theProjectDirectory=theProjectDirectory)
            
            # We check that the venv exists:
            venvsList = sh.pew("ls").split()
            print("Available venvs:")
            print(venvsList)
            self.assertTrue(venvName in venvsList)
            
            # Then we delete the venv:
            sh.pew("rm", venvName)
            print(venvName + " deleted.")
            
            # We check that the venv exists:
            venvsList = sh.pew("ls").split()
            print("Available venvs:")
            print(venvsList)


if __name__ == '__main__':
    unittest.main() # Or execute as Python unit-test in eclipse








# coding: utf-8

from workspacemanager.utils import *
import sh

def installReqs(theProjectDirectory=None):
    # Get all dirs:
    (thisLibPackageDirectory,
     theProjectDirectory,
     theProjectPackageDirectory,
     thisLibName) = getDirs(theProjectDirectory=theProjectDirectory)
    venvName = theProjectPackageDirectory.split('/')[-1] + "-venv"
    
    # Get paths:
    localDepsPath = theProjectDirectory + "/" + "local-dependencies.txt"
    reqsPath = theProjectDirectory + "/" + "requirements.txt"
    reqsPathTmp = getDir(reqsPath) + "/requirements-tmp.txt"
    
    if isFile(reqsPath):
        # Work on:
        print("pip install -r requirements.txt for " + venvName)
        # Get the requirements.txt file and prune all internal dependencies in it:
        if isFile(localDepsPath):
            
            # First we get all replacement projects
            localDeps = fileToStrList(localDepsPath)
            replacementLocalDeps = []
            for current in localDeps:
                if "/" in current:
                    replacementLocalDeps.append(current.split("/")[-1])
            reqs = fileToStrList(reqsPath)
            newReqs = []
            for current in reqs:
                if current not in replacementLocalDeps:
                    newReqs.append(current)
                else:
                    print(current + " will not be installed because a replacement project was found in local dependencies.")
            reqs = newReqs
            # Write a temp file :
            strToFile(reqs, reqsPathTmp)
            # Change the req path:
            reqsPath = reqsPathTmp
        # Install all:
        print(reqsPath)
        print(sh.pew("in", venvName, "pip", "install", "-r", reqsPath))
        # Remove if exists the tmp req:
        removeIfExists(reqsPathTmp)
    else:
        print("No requirements.txt found!")

if __name__ == '__main__':
    installReqs()
    


# coding: utf-8
import workspacemanager

from .utils import getDirs2
import os.path

def fileToStr(path):
    data = None
    with open(path, 'r') as myfile:
        data = myfile.read()
    return data

def printHelp():
    print("workspacemanager version: " + str(workspacemanager.__version__)) 
    # Get all dirs:
    (thisLibPackageDirectory,
    theProjectDirectory,
    theProjectPackageDirectory,
    thisLibName,
    workspacePath,
    theProjectName,
    thePackageName) = getDirs2()
    
    # print the README :
    readmePath = os.path.abspath(os.path.join(thisLibPackageDirectory, os.pardir)) + "/README.md"
    print(fileToStr(readmePath))

if __name__ == '__main__':
    printHelp()


# coding: utf-8

import os
from setuptools import setup, find_packages
import importlib
import re

thelibFolder = os.path.dirname(os.path.realpath(__file__))


def walklevel(some_dir, level=1):
    some_dir = some_dir.rstrip(os.path.sep)
    assert os.path.isdir(some_dir)
    num_sep = some_dir.count(os.path.sep)
    for root, dirs, files in os.walk(some_dir):
        yield root, dirs, files
        num_sep_this = root.count(os.path.sep)
        if num_sep + level <= num_sep_this:
            del dirs[:]

# We search a folder containing "__init__.py":
mainPackageName = thelibFolder.lower().split('/')[-1]
for dirname, dirnames, filenames in walklevel(thelibFolder):
    if "__init__.py" in filenames:
        print(dirname)
        print(filenames)
        mainPackageName = dirname.split("/")[-1]
print(mainPackageName)



# coding: utf-8

import unittest
import doctest
import os
from workspacemanager import setup
from workspacemanager import generateSetup
from workspacemanager.utils import *
from shutil import *
from workspacemanager.test.utils import *

# The level allow the unit test execution to choose only the top level test 
min = 0
max = 1
assert min <= max


if min <= 0 <= max:
    class DocTest(unittest.TestCase):
        def testDoctests(self):
            """Run doctests"""
            doctest.testmod(setup)

if min <= 1 <= max:
    class Test1(unittest.TestCase):
        def setUp(self):
            pass
        
        def test1(self):
            # Create a fake project:
            theProjectDirectory = createFakeDir()
            
            # Check the fake project:
            assert os.path.isdir(theProjectDirectory) is True
            
            # Generate the setup and others:
            generateSetup(theProjectDirectory=theProjectDirectory)
            
            # Check things:
            self.assertTrue("__DES" not in fileToStr(theProjectDirectory + "/setup.py"))
            self.assertTrue("<year>" not in fileToStr(theProjectDirectory + "/LICENCE.txt"))
            self.assertTrue("version" in fileToStr(theProjectDirectory + "/projecttest/__init__.py"))
        
if min <= 2 <= max:
    pass
if min <= 3 <= max:
    pass

if __name__ == '__main__':
    unittest.main() # Or execute as Python unit-test in eclipse








# coding: utf-8

# from __future__ import division, print_function, absolute_import

import os
import sys
import json
import getpass
from shutil import *
from os import listdir
from os.path import isfile, join
import datetime
from workspacemanager.utils import *

def getConf(workspacePath):
    conf = dict()
    description = ""
    confPath = workspacePath + "/wm-conf.json"
    if os.path.isfile(confPath):
        with open(confPath) as confFile:   
            try: 
                conf = json.load(confFile)
            except ValueError:
                pass
    return conf

def generateSetup(theProjectDirectory=None, userInput=True):
    # Get all dirs:
#     (thisLibPackageDirectory,
#      theProjectDirectory,
#      theProjectPackageDirectory,
#      thisLibName) = getDirs(theProjectDirectory=theProjectDirectory)
#     workspacePath = os.path.abspath(os.path.join(theProjectDirectory, os.pardir))
#     
    (thisLibPackageDirectory,
    theProjectDirectory,
    theProjectPackageDirectory,
    thisLibName,
    workspacePath,
    theProjectName,
    thePackageName,
    realPackagePath,
    realPackageName) = getDirs3(theProjectDirectory=theProjectDirectory)

    # For tests :
    if theProjectDirectory is not None:
        userInput = False
    
    # If there is a setup.py, stop here:
    if os.path.isfile(theProjectDirectory + "/setup.py"):
        print("Project already setup.")
        exit()
    
    # We check the directory structure:
    answer = None
    if userInput:
        answer = input('Do you want to check the directory structure ? Write "N" or press enter: ')
    if not (answer == "N"):
#         theFirstPackage = None
#         for (dirname, dirnames, filenames) in os.walk(theProjectDirectory):
#             theFirstPackage = dirname
#             break
#         if theFirstPackage is None:
#             print "The project must have a package folder."
#             exit()
#         if theFirstPackage != theProjectPackageDirectory:
            
#         ok = False
#         for (dirname, dirnames, filenames) in os.walk(theProjectDirectory):
#             if dirname == theProjectPackageDirectory:
#                 ok = True
#                 break
#         if not ok:
#             print "The project must have a package with the same name in lower case."
#             exit()
        if not os.path.isfile(realPackagePath + "/__init__.py"):
            print("The package of this project must have a __init__.py file.")
            exit()

    # Get all datas from the conf or the user:
    conf = getConf(workspacePath)
    if "author" not in conf or conf["author"] is None:
        author = getpass.getuser()
        authorInput = None
        if userInput:
            authorInput = input('Please write your username or press enter for "' + author + '": ')
        if authorInput is None or len(authorInput) <= 1:
            conf["author"] = author
        else:
            conf["author"] = authorInput
    if "author_email" not in conf or conf["author_email"] is None:
        conf["author_email"] = None
        if userInput:
            conf["author_email"] = input('Please write your email or press enter: ')
        if conf["author_email"] is None:
            conf["author_email"] = ""
    description = ""
    if userInput:
        description = input('Please write a description or press enter: ')
    
    # Copy datas as default in the conf:
    # Deprecated
#     answer = None
#     if userInput:
#         answer = raw_input('Do you want to keep these entries as default ? Write "Y" or press enter: ')
#     if answer == 'Y':
#         with open(confPath, "w") as confFile:
#             json.dump(conf, confFile)
    
    # Copy all file from the template:
    templatePath = thisLibPackageDirectory + "/setup-templates"
    allTemplateFiles = [f for f in listdir(templatePath) if isfile(join(templatePath, f))]
    for fileName in allTemplateFiles:
        filePath = templatePath + "/" + fileName
        filePathPaste = theProjectDirectory + "/" + fileName
        print(fileName + " created.")
        if not os.path.isfile(filePathPaste) and ".pyc" not in filePathPaste:
            copyfile(filePath, filePathPaste)

    # Replace "<year>" and "<copyright holders>":
    now = datetime.datetime.now()
    listSrc = ["<year>", "<copyright holders>"]
    listRep = [str(now.year), conf["author"]]
    replaceInFile(theProjectDirectory + "/LICENCE.txt", listSrc, listRep)
    print("LICENCE.txt updated.")
    
    # Replace datas in the setup:
    listSrc = ["__DESCRIPTION__", "__AUTHOR__", "__AUTHOR_EMAIL__"]
    listRep = [description, conf["author"], conf["author_email"]]
    replaceInFile(theProjectDirectory + "/setup.py", listSrc, listRep)
    print("setup.py updated.")
    
    # Create a requirement file if not exists:
    requPath = theProjectDirectory + "/requirements.txt"
    if not os.path.isfile(requPath):
        touch(requPath)
    print("requirements.txt created.")
    
    # If there is no __init__ or it is empty, create it with version:
    toWrite = '__version__ = "0.0.1"'
    initPath = realPackagePath + "/" + "__init__.py"
    if not os.path.isfile(initPath):
        touch(initPath)
    with open(initPath, 'w+') as f :
        filedata = f.read()
        if filedata is None or len(filedata) == 0 or filedata == "" or filedata == " ":
            f.write(toWrite)
            print("__version__ added to the __init__.py.")

if __name__ == '__main__':
    generateSetup()






        


# coding: utf-8

import unittest
import doctest
from workspacemanager.utils import *
from workspacemanager import utils
from workspacemanager.test.utils import *

# The level allow the unit test execution to choose only the top level test 
min = 0
max = 1
assert min <= max


if min <= 0 <= max:
    class DocTest(unittest.TestCase):
        def testDoctests(self):
            """Run doctests"""
            doctest.testmod(utils)

if min <= 1 <= max:
    pass


if __name__ == '__main__':
    unittest.main() # Or execute as Python unit-test in eclipse








# coding: utf-8


########## WORSPACE IN PYTHON PATH ##########
# Include all Paths :
# TODO faire un include de tous les sous dossiers du Workspace en remontant 4 dossiers
# TODO (mettre une variable pour le nombre de dossier a remonter).
# TODO mettre aussi le changement en utf-8
# TODO Et mettre le os.chdir("??") aussi
# TODO Faire une sorte d'entete pour chaque fichier python
# Updated 09/03/17
workspaceName = "Python" # Set the name of the Workspace in all ancestors folders
onlyInit = False # TODO
import sys
import os
import re
def dirsHasRegex(dirs, regex):
    if not isinstance(dirs, list):
        dirs = [dirs]
    # for all subdirs:
    for currentDir in dirs:
        # We get all files:
        files = getFileNames(currentDir)
        # We check if there .py or __init__.py:
        for fname in files:
            if re.search(regex, fname) is not None:
                return True
    return False

def getFileNames(currentFolder):
    files = [item for item in os.listdir(currentFolder) if os.path.isfile(os.path.join(currentFolder, item))]
    return files

def getSubdirsPaths(currentFolder):
    dirs = [currentFolder + item + "/" for item in os.listdir(currentFolder) if os.path.isdir(os.path.join(currentFolder, item))]
    return dirs

# We get the Workspace dir:
currentFolder = os.path.dirname(os.path.abspath("__file__")) # If the script is executed in an other script
# currentFolder = os.path.dirname(os.path.abspath(__file__)) # If this is pasted in the main script
workspace = currentFolder.split(workspaceName)[0] + workspaceName + "/"
# And while there are folders to watch:
penddingFolderList = [workspace]
pathsToAdd = []
while len(penddingFolderList) != 0:
    # We pop the current folder in in the pending queue:
    currentFolder = penddingFolderList.pop(0)
    if not currentFolder.split("/")[-2].startswith("."):
        hasPy = dirsHasRegex(currentFolder, ".py$")
        atLeastOneSubDirHasInit = dirsHasRegex(getSubdirsPaths(currentFolder), "__init__.py")
        # We add it in the path list:
        if hasPy or atLeastOneSubDirHasInit:
            pathsToAdd.append(currentFolder)
        # And We add the subfolders in the pending queue if there is no __init__.py:
        if not atLeastOneSubDirHasInit:
            subDirs = getSubdirsPaths(currentFolder)
            penddingFolderList += subDirs
# We add all paths in the python path:
for path in pathsToAdd:
    sys.path.append(path)
#     print path
#############################################


# coding: utf-8

import os
from shutil import *
from workspacemanager.utils import *
import sh

def fileToStr(path):
    with open(path, 'r') as myfile:
        data = myfile.read()
    return data

def createFakeDir(projectName="ProjectTest", resetWorkspaceTest=True, parentFolder=None):
    """ Create a fake project """
    if parentFolder is None:
        parentFolder = ""
    else:
        parentFolder = parentFolder + "/"
    projectPackage = projectName.lower()
    thisLibPackageDirectory = os.path.dirname(os.path.realpath(__file__))
    thisLibPackageDirectory = os.path.abspath(os.path.join(thisLibPackageDirectory, os.pardir))
    workspaceTestPath = thisLibPackageDirectory + '/test/workspacetest/'
    if resetWorkspaceTest and os.path.isdir(workspaceTestPath):
        rmtree(workspaceTestPath)
    if resetWorkspaceTest:
        os.mkdir(workspaceTestPath)
    if parentFolder != "" and not os.path.isdir(workspaceTestPath + "/" + parentFolder):
        os.mkdir(workspaceTestPath + "/" + parentFolder) # Warning don't work if parentFolder's depth > 1
    sh.touch(workspaceTestPath + "/wm-conf.json")
    theProjectDirectory = workspaceTestPath + '/' + parentFolder + projectName
    theProjectPackageDirectory = theProjectDirectory + '/' + projectPackage
    os.mkdir(theProjectDirectory)
    os.mkdir(theProjectPackageDirectory)
    touch(theProjectPackageDirectory + "/__init__.py")
    return theProjectDirectory

if __name__ == '__main__':
    pass


# coding: utf-8
# cd /home/hayj/Workspace/Python/Organization/WorkspaceManager && sudo python ./workspacemanager/test/depstest.py
# pew rm dep1-venv dep3-venv dep2-venv projecttest-venv projecttest1-venv
# pew rm p1-venv p2-venv p3-venv p4-venv p5-venv

import unittest
import doctest
from workspacemanager import deps
from workspacemanager import installDeps
from workspacemanager import generateSetup
from workspacemanager import generateVenv
from workspacemanager.utils import *
from workspacemanager.test.utils import *
import sh
import glob

# The level allow the unit test execution to choose only the top level test 
min = 3
max = 3
assert min <= max


# WARNING : to execute each tests alone
if min != max:
    print("WARNING : you have to execute each tests alone")
    exit()

if min <= 0 <= max:
    class DocTest(unittest.TestCase):
        def testDoctests(self):
            """Run doctests"""
            doctest.testmod(deps)

if min <= 1 <= max:
    class Test1(unittest.TestCase):
        def test1(self):
            # Create a fake project:
            projectName = "ProjectTest"
            theProjectDirectory = createFakeDir(projectName=projectName)
            dep1Path = createFakeDir(projectName="Dep1", resetWorkspaceTest=False)
            dep2Path = createFakeDir(projectName="Dep2", resetWorkspaceTest=False)
            allProjectsPaths = [theProjectDirectory, dep1Path, dep2Path]
            allProjectsPackages = []
            for current in allProjectsPaths:
                allProjectsPackages.append(current.split('/')[-1].lower())
            
            # Check the fake project:
            assert os.path.isdir(theProjectDirectory) is True
            
            # Create setups:
            for current in allProjectsPaths:
                print("Setting up of " + current)
                generateSetup(current)
            
            # Create venvs:
            for current in allProjectsPaths:
                generateVenv(current)
            
            # Check if Dep1 and Dep2 are not in the pip freeze:
            pipFreeze = sh.pew("in", projectName.lower() + "-venv", "pip", "freeze")
            self.assertTrue("dep1" not in pipFreeze)
            self.assertTrue("dep2" not in pipFreeze)
            
            # Installation:
            with open(theProjectDirectory + "/local-dependencies.txt", "w") as f:
                f.write("Dep1\nDep2\n")
            installDeps(theProjectDirectory=theProjectDirectory)
            
            # Check if Dep1 and Dep2 are in the pip freeze:
            pipFreeze = sh.pew("in", projectName.lower() + "-venv", "pip", "freeze")
            self.assertTrue("dep1" in pipFreeze)
            self.assertTrue("dep2" in pipFreeze)
            
            # Remove all venvs:
            for current in allProjectsPackages:
                venvName = current + "-venv"
                print("Deletion of " + venvName)
                sh.pew("rm", venvName)


if min <= 2 <= max:
    class TestRecursiveCircular(unittest.TestCase):
        def test1(self):
            # Create a fake project:
            projectName = "ProjectTest"
            theProjectDirectory = createFakeDir(projectName=projectName)
            dep1Path = createFakeDir(projectName="Dep1", resetWorkspaceTest=False)
            dep2Path = createFakeDir(projectName="Dep2", resetWorkspaceTest=False)
            dep3Path = createFakeDir(projectName="Dep3", resetWorkspaceTest=False)
            allProjectsPaths = [theProjectDirectory, dep1Path, dep2Path, dep3Path]
            allProjectsPackages = []
            for current in allProjectsPaths:
                allProjectsPackages.append(current.split('/')[-1].lower())
            
            # Create setups:
            for current in allProjectsPaths:
                print("Setting up of " + current)
                generateSetup(current)
            
            # Create venvs:
            for current in allProjectsPaths:
                generateVenv(current)
            
            # Check if Dep1 and Dep2 are not in the pip freeze:
            pipFreeze = sh.pew("in", projectName.lower() + "-venv", "pip", "freeze")
            self.assertTrue("dep1" not in pipFreeze)
            self.assertTrue("dep2" not in pipFreeze)
            self.assertTrue("dep3" not in pipFreeze)
            
            # Create dependencies files:
            def createDependency(projectPath, dependenciesNames=[]):
                with open(projectPath + "/local-dependencies.txt", "w") as f:
                    f.write("\n".join(dependenciesNames))
            createDependency(theProjectDirectory, ["Dep1"])
            createDependency(dep1Path, ["Dep2"])
            createDependency(dep2Path, ["Dep3", projectName])
            
            # Install:
            installDeps(theProjectDirectory=theProjectDirectory)
            
            # Check if Dep1 and Dep2 are in the pip freeze:
            pipFreeze = sh.pew("in", projectName.lower() + "-venv", "pip", "freeze")
            self.assertTrue("dep1" in pipFreeze)
            self.assertTrue("dep2" in pipFreeze)
            self.assertTrue("dep3" in pipFreeze)
            self.assertTrue(projectName.lower() not in pipFreeze)
            
            # Remove all venvs:
            for current in allProjectsPackages:
                venvName = current + "-venv"
                print("Deletion of " + venvName)
                sh.pew("rm", venvName)

if min <= 3 <= max:
    class TestReq(unittest.TestCase):
        def test1(self):
            # Create a fake project:
            projectName = "ProjectTest"
            theProjectDirectory = createFakeDir(projectName=projectName)
            dep1Path = createFakeDir(projectName="Dep1", resetWorkspaceTest=False)
            allProjectsPaths = [theProjectDirectory, dep1Path]
            allProjectsPackages = []
            for current in allProjectsPaths:
                allProjectsPackages.append(current.split('/')[-1].lower())
            
            # Create setups:
            for current in allProjectsPaths:
                print("Setting up of " + current)
                generateSetup(current)
            
            # Create venvs:
            for current in allProjectsPaths:
                generateVenv(current)
            
            # Check if Dep1 and Dep2 are not in the pip freeze:
            pipFreeze = sh.pew("in", projectName.lower() + "-venv", "pip", "freeze")
            self.assertTrue("dep1" not in pipFreeze)
            self.assertTrue("markdown" not in pipFreeze)
            self.assertTrue("PyUserInput" not in pipFreeze)
            
            # Add markdown as deps1's requirement:
            with open(dep1Path + "/requirements.txt", "w") as f:
                f.write("markdown")
                f.write("\n")
#                 f.write("-e git://github.com/PyUserInput/PyUserInput.git#egg=PyUserInput")
                f.write("https://github.com/PyUserInput/PyUserInput/zipball/master#egg=PyUserInput")
            
            # Create dependencies files:
            def createDependency(projectPath, dependenciesNames=[]):
                with open(projectPath + "/local-dependencies.txt", "w") as f:
                    f.write("\n".join(dependenciesNames))
            createDependency(theProjectDirectory, ["Dep1"])
            
            # Install:
            installDeps(theProjectDirectory=theProjectDirectory)
            
            # Check if Dep1 and Dep2 are in the pip freeze:
            pipFreeze = sh.pew("in", projectName.lower() + "-venv", "pip", "freeze")
            self.assertTrue("dep1" in pipFreeze)
            self.assertTrue("Markdown" in pipFreeze)
            self.assertTrue("PyUserInput" in pipFreeze)
            
            # Remove all venvs:
            for current in allProjectsPackages:
                venvName = current + "-venv"
                print("Deletion of " + venvName)
                sh.pew("rm", venvName)
            

if min <= 4 <= max:
    class TestTree(unittest.TestCase):
        def test1(self):
            # Create a fake project:
            p1 = createFakeDir(projectName="P1", resetWorkspaceTest=True)
            p2 = createFakeDir(projectName="P2", resetWorkspaceTest=False)
            p3 = createFakeDir(projectName="P3", resetWorkspaceTest=False, parentFolder="Group")
            p4 = createFakeDir(projectName="P4", resetWorkspaceTest=False, parentFolder="Group")
            p5 = createFakeDir(projectName="P5", resetWorkspaceTest=False, parentFolder="Group/SubGroup")
            allProjectsPaths = [p1, p2, p3, p4, p5]
            allProjectsPackages = []
            for current in allProjectsPaths:
                allProjectsPackages.append(current.split('/')[-1].lower())
            
            # Create setups:
            for current in allProjectsPaths:
                print("Setting up of " + current)
                generateSetup(current)
#             
#             # Create venvs:
            for current in allProjectsPaths:
                generateVenv(current)
             
            # Check if Dep1 and Dep2 are not in the pip freeze:
            pipFreeze = sh.pew("in", "p1" + "-venv", "pip", "freeze")
            self.assertTrue("p2" not in pipFreeze)
             
            # Add markdown as deps1's requirement:
            with open(p1 + "/requirements.txt", "w") as f:
                f.write("markdown")
                f.write("\n")
             
            # Create dependencies files:
            def createDependency(projectPath, dependenciesNames=[]):
                with open(projectPath + "/local-dependencies.txt", "w") as f:
                    f.write("\n".join(dependenciesNames))
            createDependency(p1, ["P2", "P3", "P4", "P5"])
            createDependency(p5, ["P1", "P2", "P3", "P4"])
             
            # Install:
            print("------- Installation of P1's deps -------")
            installDeps(theProjectDirectory=p1)
            print("------- Installation of P5's deps -------")
            installDeps(theProjectDirectory=p5)
             
            # Check if Dep1 and Dep2 are in the pip freeze:
            pipFreeze = sh.pew("in", "p1" + "-venv", "pip", "freeze")
            print("------ pip freeze de p1 -------")
            print(pipFreeze)
            self.assertTrue("p2" in pipFreeze)
            self.assertTrue("p3" in pipFreeze)
            self.assertTrue("p4" in pipFreeze)
            self.assertTrue("p5" in pipFreeze)
            pipFreeze = sh.pew("in", "p5" + "-venv", "pip", "freeze")
            print("------ pip freeze de p5 -------")
            print(pipFreeze)
            self.assertTrue("p2" in pipFreeze)
            self.assertTrue("p3" in pipFreeze)
            self.assertTrue("p4" in pipFreeze)
            self.assertTrue("p1" in pipFreeze)
            self.assertTrue("Markdown" in pipFreeze)
            
             
            # Remove all venvs:
            for current in allProjectsPackages:
                venvName = current + "-venv"
                print("Deletion of " + venvName)
                sh.pew("rm", venvName)
            
if __name__ == '__main__':
    unittest.main() # Or execute as Python unit-test in eclipse








# coding: utf-8

from workspacemanager.utils import *
import time
import sh


def generateVenv(theProjectDirectory=None):
    # Get args:
    argv = argvOptionsToDict()
    if argv is None and theProjectDirectory is None:
        print("Please check the readme for the command usage.")
        exit()
    
    # Get all dirs:
    (thisLibPackageDirectory,
    theProjectDirectory,
    theProjectPackageDirectory,
    thisLibName,
    workspacePath,
    theProjectName,
    thePackageName,
    realPackagePath,
    realPackageName) = getDirs3(theProjectDirectory=theProjectDirectory)
         
    # Check if the venv exists:
    venvName = thePackageName + "-venv"
    venvsList = sh.pew("ls").split()
    if venvName in venvsList:
        print(venvName + " already exists.")
    else:
        # Build the command:
        commandOptions = "-a " + theProjectDirectory + " " + venvName
        if argv is not None and "p" in argv:
            commandOptions = "-p " + argv["p"] + " " + commandOptions
        commandOptions = "new -d " + commandOptions # -d prevent the newly created venv activation
        
        # Execute the command:
        print(sh.pew(*commandOptions.split()))
    
    

if __name__ == '__main__':
    pass
    
#     generateVenv()















# coding: utf-8

import os
from setuptools import setup, find_packages
import importlib
import re

# Vars to set:
description = "__DESCRIPTION__"
author = "__AUTHOR__"
author_email = "__AUTHOR_EMAIL__"
version = "0.0.1" # replaced by the version in the main init file if exists

# Current dir:
thelibFolder = os.path.dirname(os.path.realpath(__file__))

# We take all requirements from the file or you can set it here :
requirementPath = thelibFolder + '/requirements.txt'
install_requires = [] # Example : ["gunicorn", "docutils >= 0.3", "lxml==0.5a7"]
dependency_links = []
if os.path.isfile(requirementPath):
    with open(requirementPath) as f:
        dependency_links = []
        install_requires = []
        required = f.read().splitlines()
        for current in required:
            if 'git' in current:
                if "https" not in current:
                    current = current.replace("-e git", "https")
                    current = current.replace(".git#egg", "/zipball/master#egg")
                dependency_links.append(current)
            else:
                install_requires.append(current)

# dependency_links is deprecated, see https://serverfault.com/questions/608192/pip-install-seems-to-be-ignoring-dependency-links
dependency_links = []

# We search a folder containing "__init__.py":
def walklevel(some_dir, level=1):
    some_dir = some_dir.rstrip(os.path.sep)
    assert os.path.isdir(some_dir)
    num_sep = some_dir.count(os.path.sep)
    for root, dirs, files in os.walk(some_dir):
        yield root, dirs, files
        num_sep_this = root.count(os.path.sep)
        if num_sep + level <= num_sep_this:
            del dirs[:]
mainPackageName = thelibFolder.lower().split('/')[-1]
for dirname, dirnames, filenames in walklevel(thelibFolder):
    if "__init__.py" in filenames:
        mainPackageName = dirname.split("/")[-1]
packagePath = thelibFolder + '/' + mainPackageName
# Get the version of the lib in the __init__.py:
initFilePath = packagePath + '/' + "__init__.py"
if os.path.isdir(packagePath):
    with open(initFilePath, 'r') as f:
        text = f.read()
        result = re.search('^__version__\s*=\s*["\'](.*)["\']', text, flags=re.MULTILINE)
        if result is not None:
            version = result.group(1)

# To import the lib, use:
# thelib = importlib.import_module(mainPackageName)

# Readme content:
readme = None
readmePath = thelibFolder + '/README.md'
if os.path.isfile(readmePath):
    try:
        import pypandoc
        readme = pypandoc.convert(readmePath, 'rst')
    except(IOError, ImportError) as e:
        print(e)
        print("Cannot use pypandoc to convert the README...")
        readme = open(readmePath).read()

packageName = thelibFolder.lower().split('/')[-1]  
if packageName.startswith("pip-"):
    packageName = mainPackageName.lower()

# The whole setup:
setup(

    # The name for PyPi:
    name=packageName,

    # The version of the code which is located in the main __init__.py:
    version=version,

    # All packages to add:
    packages=find_packages(),

    # About the author:
    author=author,
    author_email=author_email,

    # A short desc:
    description=description,

    # A long desc with the readme:
    long_description=readme,

    # Dependencies:
    install_requires=install_requires,
    dependency_links=dependency_links,
    
    # For handle the MANIFEST.in:
    include_package_data=True,

    # The url to the official repo:
    # url='https://',

    # You can choose what you want here : https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 1 - Planning",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Topic :: Utilities",
    ],

    # If you want a command line like "do-something", on a specific funct of the package :
#     entry_points = {
#         'console_scripts': [
#             'wm-setup = workspacemanager.setup:generateSetup',
#             'wm-pew = workspacemanager.venv:generateVenv',
#             'wm-deps = workspacemanager.deps:installDeps',
#         ],
#     },
)










# coding: utf-8

from workspacemanager.utils import *
import sh

def dispFreeze(theProjectDirectory=None):
    # Get all dirs:
    (thisLibPackageDirectory,
     theProjectDirectory,
     theProjectPackageDirectory,
     thisLibName) = getDirs(theProjectDirectory=theProjectDirectory)
    venvName = theProjectPackageDirectory.split('/')[-1] + "-venv"
    
    # Work on:
    print("pip freeze for " + venvName)
    print(sh.pew("in", venvName, "pip", "freeze"))


if __name__ == '__main__':
    dispFreeze()
    


# coding: utf-8

import os, errno
import sys
import subprocess
import glob

def touch(fname, times=None):
    with open(fname, 'a'):
        os.utime(fname, times)

def replaceInFile(path, listSrc, listRep):
    with open(path, 'r') as f :
        filedata = f.read()
    for i in range(len(listSrc)):
        src = listSrc[i]
        rep = listRep[i]
        filedata = filedata.replace(src, rep)
    with open(path, 'w') as f:
        f.write(filedata)


def getDirs(theProjectDirectory=None):
    # Get args:
    argv = argvOptionsToDict()
    
    # Get all dirs:
    thisLibPackageDirectory = os.path.dirname(os.path.realpath(__file__))
    if theProjectDirectory is None and argv is not None and "a" in argv:
        theProjectDirectory = argv["a"]
    if theProjectDirectory is None:
        theProjectDirectory = os.getcwd()
    if not os.path.isdir(theProjectDirectory):
        print(theProjectDirectory + " is not a directory.")
        exit()
    theProjectPackageDirectory = theProjectDirectory + "/" + theProjectDirectory.split("/")[-1].lower()
    thisLibName = thisLibPackageDirectory.split("/")[-1].lower()
    # thisLibRootDirectory = os.path.abspath(os.path.join(thisLibPackageDirectory, os.pardir))
    
    return (thisLibPackageDirectory, theProjectDirectory, theProjectPackageDirectory, thisLibName)

def getDirs2(theProjectDirectory=None):
    # Get args:
    argv = argvOptionsToDict()
    
    # Get all dirs:
    thisLibPackageDirectory = os.path.dirname(os.path.realpath(__file__))
    if theProjectDirectory is None and argv is not None and "a" in argv:
        theProjectDirectory = argv["a"]
    if theProjectDirectory is None:
        theProjectDirectory = os.getcwd()
    if not os.path.isdir(theProjectDirectory):
        print(theProjectDirectory + " is not a directory.")
        exit()
    theProjectPackageDirectory = theProjectDirectory + "/" + theProjectDirectory.split("/")[-1].lower()
    thisLibName = thisLibPackageDirectory.split("/")[-1].lower()
    # thisLibRootDirectory = os.path.abspath(os.path.join(thisLibPackageDirectory, os.pardir))
    
    workspacePath = theProjectDirectory
    while not os.path.isfile(workspacePath + "/wm-conf.json"):
        workspacePath = os.path.abspath(os.path.join(workspacePath, os.pardir))
    theProjectName = theProjectDirectory.split('/')[-1]
    thePackageName = theProjectDirectory.split('/')[-1].lower()
    
    return (thisLibPackageDirectory,
            theProjectDirectory,
            theProjectPackageDirectory,
            thisLibName,
            workspacePath,
            theProjectName,
            thePackageName)

def getDirs3(theProjectDirectory=None):
    (thisLibPackageDirectory,
    theProjectDirectory,
    theProjectPackageDirectory,
    thisLibName,
    workspacePath,
    theProjectName,
    thePackageName) = getDirs2(theProjectDirectory=theProjectDirectory)
    
    # We searh for a correct package name and get the first folder:
    firstPackage = None
    currentPackage = None
    correctPackageNameFound = False
    for (dirname, dirnames, filenames) in os.walk(theProjectDirectory):
        if dirname != theProjectDirectory and dirname != theProjectDirectory + "/.settings":
            currentPackage = dirname
            if firstPackage is None:
                firstPackage = dirname
            if currentPackage == theProjectPackageDirectory:
                correctPackageNameFound = True
                break

    # We set the real path of the package:
    realPackagePath = None
    if correctPackageNameFound:
        realPackagePath = theProjectPackageDirectory
    else:
        realPackagePath = firstPackage
    
    # We get the name of the apckage:
    realPackageName = realPackagePath.split("/")[-1]
    
    return (thisLibPackageDirectory,
            theProjectDirectory,
            theProjectPackageDirectory,
            thisLibName,
            workspacePath,
            theProjectName,
            thePackageName,
            realPackagePath,
            realPackageName)

def getVenvName(theProjectName):
    return theProjectName.lower() + "-venv"

class GlobSortEnum():
    (
        MTIME,
        NAME,
        SIZE
    ) = list(range(3))

def sortedGlob(regex, caseSensitive=True, sortBy=GlobSortEnum.NAME, reverse=False):
    # case insensitive glob function :
    def insensitiveGlob(pattern):
        def either(c):
            return '[%s%s]' % (c.lower(), c.upper()) if c.isalpha() else c

        return glob.glob(''.join(map(either, pattern)))

    # Handle case insentive param :
    if caseSensitive:
        paths = glob.glob(regex)
    else:
        paths = insensitiveGlob(regex)

    # Sort the result :
    if sortBy == GlobSortEnum.NAME:
        paths.sort(reverse=reverse)
    elif sortBy == GlobSortEnum.MTIME:
        paths.sort(key=os.path.getmtime, reverse=reverse)
    elif sortBy == GlobSortEnum.SIZE:
        paths.sort(key=os.path.getsize, reverse=reverse)

    return paths

def findProject(workspacePath, projectName):
    # Find all dirs according to the project name:
    projectPaths = []
    for root, dirnames, _ in os.walk(workspacePath):
        if projectName in dirnames:
            projectPaths.append(root + "/" + projectName)
    # Remove all eclipse folders:
    newProjectPaths = []
    for current in projectPaths:
        if ".metadata" not in current:
            newProjectPaths.append(current)
    projectPaths = newProjectPaths
    # Check wether there is one dir:
    if projectPaths is None or len(projectPaths) == 0:
        print(projectName + " not found.")
        return None
    if len(projectPaths) > 1:
        print(projectName + " conflicts.")
        return None
    # Return the path:
    return projectPaths[0]

def argvOptionsToDict(argv=None):
    """
        This function convert a command in dict key values according to command options.
        If the function return None, it means the argv doesn't have a good format.
        
        :example:
        >>> argvOptionsToDict(argv=["thecommand", "-r", "r", "-a", "a"])
        {'a': 'a', 'r': 'r', 'command': 'thecommand'}
        >>> argvOptionsToDict(argv=["thecommand", "r", "r"]) is None
        True
        >>> argvOptionsToDict(argv=["thecommand"])
        {'command': 'thecommand'}
        >>> argvOptionsToDict(argv=["thecommand", "r"]) is None
        True
        >>> argvOptionsToDict(argv=["thecommand", "--abcd", "/abcd/e"])
        {'abcd': '/abcd/e', 'command': 'thecommand'}
    """
    if argv is None:
        argv = sys.argv
    argvDict = dict()
    if argv is None or len(argv) == 0 or len(argv) % 2 == 0:
        return None
    argvDict["command"] = argv[0]
    for i in range(1, len(argv), 2):
        current = argv[i]
        if len(current) == 2:
            if not current.startswith('-'):
                return None
            argvDict[str(current[1])] = argv[i + 1]
        elif len(current) >= 3:
            if not current.startswith('--'):
                return None
            argvDict[str(current[2:len(current)])] = argv[i + 1]
        else:
            return None
    return argvDict

def getDir(filePath):
    return os.path.dirname(os.path.abspath(filePath))

def getCurrentDir():
    return os.getcwd()

def pathToAbsolute(path):
    if len(path) > 0 and path[0] != "/":
        path = getCurrentDir() + "/" + path
    return path

def isFile(filePath):
    filePath = pathToAbsolute(filePath)
#     print filePath
    return os.path.isfile(filePath)

def fileToStr(path):
    with open(path, 'r') as myfile:
        data = myfile.read()
    return data

def fileToStrList(path, strip=True):
    data = fileToStr(path)
    if strip:
        data = data.strip()
    return data.splitlines()

def strToFile(text, path):
#     if not isDir(getDir(path)) and isDir(getDir(text)):
#         path, text = text, path
    if isinstance(text, list):
        text = "\n".join(text)
    textFile = open(path, "w")
    textFile.write(text)
    textFile.close()

def removeIfExists(path):
    try:
        os.remove(path)
    except OSError as e: # this would be "except OSError, e:" before Python 2.6
        if e.errno != errno.ENOENT: # errno.ENOENT = no such file or directory
            raise # re-raise exception if a different error occurred

if __name__ == '__main__':
    print(fileToStrList("/home/hayj/test.txt"))










# coding: utf-8
# cd /home/hayj/Workspace/Python/Organization/WorkspaceManager && sudo python ./workspacemanager/test/disttest.py
# pew rm dep1-venv dep3-venv dep2-venv projecttest-venv projecttest1-venv
# pew rm p1-venv p2-venv p3-venv p4-venv p5-venv

# sudo chmod -R 777 /home/hayj/Workspace/Python/Organization/WorkspaceManager/workspacemanager/test/workspacetest
# /home/hayj/Workspace/Python/Organization/WorkspaceManager/workspacemanager/test/workspacetest/P1/wm-dist/rsync-all.sh

import os
exec(compile(open(os.path.dirname(os.path.abspath(__file__)) + "/setpythonpath.py").read(), os.path.dirname(os.path.abspath(__file__)) + "/setpythonpath.py", 'exec'), {})


import unittest
import doctest
from workspacemanager import dist
from workspacemanager import getDependencies
from workspacemanager import generateDists
from workspacemanager import generateSetup
from workspacemanager import generateVenv
from workspacemanager.utils import *
from workspacemanager.test.utils import *
import sh
import glob

# The level allow the unit test execution to choose only the top level test 
min = 1
max = 1
assert min <= max


# WARNING : to execute each tests alone
if min != max:
    print("WARNING : you have to execute each tests alone")
    exit()

if min <= 0 <= max:
    class DocTest(unittest.TestCase):
        def testDoctests(self):
            """Run doctests"""
            doctest.testmod(dist)

if min <= 1 <= max:
    class Test1(unittest.TestCase):
        def test1(self):
            
            jump = False
            
            if jump:
                p1 = "/home/hayj/Workspace/Python/Organization/WorkspaceManager/workspacemanager/test/workspacetest/P1"
            else:
                # Create a fake project:
                p1 = createFakeDir(projectName="P1", resetWorkspaceTest=True)
                p2 = createFakeDir(projectName="P2", resetWorkspaceTest=False)
                p3 = createFakeDir(projectName="P3", resetWorkspaceTest=False, parentFolder="Group")
                p4 = createFakeDir(projectName="P4", resetWorkspaceTest=False, parentFolder="Group")
                p5 = createFakeDir(projectName="P5", resetWorkspaceTest=False, parentFolder="Group/SubGroup")
                allProjectsPaths = [p1, p2, p3, p4, p5]
                allProjectsPackages = []
                for current in allProjectsPaths:
                    allProjectsPackages.append(current.split('/')[-1].lower())
                
                # Create setups:
                for current in allProjectsPaths:
                    print("Setting up of " + current)
                    generateSetup(current)
    #             
    #             # Create venvs:
                for current in allProjectsPaths:
                    generateVenv(current)
                 
                # Check if Dep1 and Dep2 are not in the pip freeze:
                pipFreeze = sh.pew("in", "p1" + "-venv", "pip", "freeze")
                self.assertTrue("p2" not in pipFreeze)
                 
                # Add markdown as deps1's requirement:
                with open(p1 + "/requirements.txt", "w") as f:
                    f.write("markdown")
                    f.write("\n")
             
                # Create dependencies files:
                def createDependency(projectPath, dependenciesNames=[]):
                    with open(projectPath + "/local-dependencies.txt", "w") as f:
                        f.write("\n".join(dependenciesNames))
                createDependency(p1, ["P2", "P3", "P4", "P5"])
                createDependency(p5, ["P1", "P2", "P3", "P4"])
             
            # Tests:
            print("Dependecies test...")
#             for p in getDependencies(theProjectDirectory=p1): print p
            generateDists(theProjectDirectory=p1)
            
             
            # Remove all venvs:
            for current in allProjectsPackages:
                venvName = current + "-venv"
                print("Deletion of " + venvName)
                sh.pew("rm", venvName)
            
            # TODO
            
if __name__ == '__main__':
    unittest.main() # Or execute as Python unit-test in eclipse








__version__ = "0.2.13"

from .setup import generateSetup
from .venv import generateVenv
from .deps import installDeps
from .workon import dispWorkon
from .freeze import dispFreeze
from .req import installReqs
from .dist import getDependencies
from .dist import generateDists
from .help import printHelp
from .utils import *





# coding: utf-8

import os
from setuptools import setup, find_packages
import importlib
import re

# Vars to set:
description = "This tool can manage a workspace by providing some useful functions (generation of the setup file and others, generate a virtual env according to the project name, install your own dependencies, generate dist of all dependencies...). See the readme for more informations."
author = "hayj"
author_email = "hj@hayj.fr"
version = "0.0.1" # replaced by the version in the main init file if exists

# We take all requirements from the file or you can set it here :
requirementPath = 'requirements.txt'
install_requires = None # Example : ["gunicorn", "docutils >= 0.3", "lxml==0.5a7"]
dependency_links = None
if install_requires is None and dependency_links is None and os.path.isfile(requirementPath):
    with open(requirementPath) as f:
        dependency_links = []
        install_requires = []
        required = f.read().splitlines()
        for current in required:
            if 'git' in current:
                dependency_links.append(current)
            else:
                install_requires.append(current)

# Get the version of the lib in the __init__.py:
thelibFolder = os.path.dirname(os.path.realpath(__file__))
mainPackageName = thelibFolder.lower().split('/')[-1]
packagePath = thelibFolder + '/' + mainPackageName
initFilePath = packagePath + '/' + "__init__.py"
if os.path.isdir(packagePath):
    with open(initFilePath, 'r') as f:
        text = f.read()
        result = re.search('^__version__\s*=\s*["\'](.*)["\']', text, flags=re.MULTILINE)
        if result is not None:
            version = result.group(1)

# To import the lib, use:
# thelib = importlib.import_module(mainPackageName)

# Readme content:
readme = None
readmePath = 'README.md'
if os.path.isfile(readmePath):
    try:
        import pypandoc
        readme = pypandoc.convert(readmePath, 'rst')
    except(IOError, ImportError):
        readme = open('README.md').read()

# The whole setup:
setup(

    # The name for PyPi:
    name="workspacemanager",

    # The version of the code which is located in the main __init__.py:
    version=version,

    # All packages to add:
    packages=find_packages(),

    # About the author:
    author=author,
    author_email=author_email,

    # A short desc:
    description=description,

    # A long desc with the readme:
    long_description=readme,

    # Dependencies:
    install_requires=install_requires,
    dependency_links=dependency_links,
    
    # For handle the MANIFEST.in:
    include_package_data=True,

    # The url to the official repo:
    # url='https://',

    # You can choose what you want here : https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 1 - Planning",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Topic :: Utilities",
    ],

    # If you want a command line like "do-something", on a specific funct of the package :
    entry_points = {
        'console_scripts': [
            'wm-setup = workspacemanager.setup:generateSetup',
            'wm-pew = workspacemanager.venv:generateVenv',
            'wm-deps = workspacemanager.deps:installDeps',
            'wm-workon = workspacemanager.workon:dispWorkon',
            'wm-freeze = workspacemanager.freeze:dispFreeze',
            'wm-req = workspacemanager.req:installReqs',
            'wm-dist = workspacemanager.dist:generateDists',
            'wm-help = workspacemanager.help:printHelp',
        ],
    },
)







# coding: utf-8

from workspacemanager.utils import *
import sh

def dispWorkon(theProjectDirectory=None):
    # Get all dirs:
    (thisLibPackageDirectory,
    theProjectDirectory,
    theProjectPackageDirectory,
    thisLibName,
    workspacePath,
    theProjectName,
    thePackageName,
    realPackagePath,
    realPackageName) = getDirs3(theProjectDirectory=theProjectDirectory)
    venvName = thePackageName + "-venv"
    
    # Work on:
    print("pew workon " + venvName)


if __name__ == '__main__':
    dispWorkon()
    